@@ -39,6 +39,27 @@ class Verb(pylatex.base_classes.Environment):
     _latex_name="verbatim"
     content_separator=""
 
+class Minipage(pylatex.base_classes.Environment):
+    _repr_attributes_mapping = {
+        "width": "arguments",
+    }
+
+    _latex_name="minipage"
+
+    def __init__(self, width):
+        self.width = width
+        super().__init__(arguments=[width])
+
+class Hspace(pylatex.base_classes.CommandBase):
+    _repr_attributes_mapping = {
+        "width": "arguments",
+    }
+
+    _latex_name="hspace"
+
+    def __init__(self, width):
+        self.width = width
+        super().__init__(arguments=[width])
 
 class Href(pylatex.base_classes.CommandBase):
     _repr_attributes_mapping = {
@@ -153,11 +174,14 @@ class Parbox(pylatex.base_classes.CommandBase):
 class Tcolorbox(pylatex.base_classes.Environment):
     _repr_attributes_mapping = {
         "text": "arguments",
+        "options": "options"
     }
-    packages = [Package("tcolorbox")]
+    packages = [Package("xcolor", options=["usenames", "dvipsnames"]), Package("tcolorbox")]
 
-    def __init__(self, text):
-        super().__init__(arguments=[text])   
+    def __init__(self, text, options=None):
+        if(options != None):
+            options = NoEscape(options)
+        super().__init__(arguments=[text], options=options)   
 
 def strikethrough(text):
     """Needs soul package"""
@@ -183,15 +207,15 @@ def _random_image_path() -> str:
     random_file += ".png"
     return random_file
 
-def _load_image(url) -> str:
+def _load_image(url, path) -> str:
     if(len(url) <= 8 or url[:8] != "https://"):
         url = _kattis_url() + url
 
-    path = _random_image_path()
+    path = Path(path, _random_image_path())
     request = requests.get(url, stream=True)
     request.raise_for_status()
     with open(path, 'wb') as file:
-        for chunk in request:
+        for chunk in request.iter_content(1024):
             file.write(chunk)
 
     _files_to_remove.append(path)
@@ -279,6 +303,23 @@ def _generate_header(problem: str, document: Document, parser: BeautifulSoup):
     document.change_document_style(style)
     return metadata
 
+def _generate_minipage(document, element, path):
+    width = element.get("style")
+    start_index = width.find("width:") + 6
+    end_index = width[start_index:].find(" ")
+    if(end_index == -1):
+        width = NoEscape(width[start_index:])
+
+    else:
+        end_index += start_index
+        width = NoEscape(width[start_index:end_index])
+
+    minipage = Minipage(width)
+    _process_content(minipage, element, path)
+    document.append(minipage)
+    document.append(NoEscape(r"\vskip0pt"))
+
+
 def _generate_pre(document, element):
     text = NoEscape(element.get_text())
     enviroment = Verb()
@@ -302,7 +343,7 @@ def _process_text(tag):
             delimiter="^"
         if delimiter == "^" and tag_text.find("^") != -1:
             delimiter="<"
-        return verbatim(tag_text, delimiter=delimiter)
+        return verbatim(tag_text.replace("\n", " "), delimiter=delimiter)
     elif(tag.name == "a"):
         return _get_href(tag)
     elif(tag.name == "del"):
@@ -310,14 +351,14 @@ def _process_text(tag):
     else:
         return tag_text.replace("\n", " ")
 
-def _generate_paragraph(document, element):
+def _generate_paragraph(document, element, path):
     if(element.name == "p"):
         document.append(NewParagraph())
     text = ""
     for tag in element:
         if(tag != None and tag.name == "img"):
             document.append(NoEscape(text))
-            _generate_image(document, tag)
+            _generate_image(document, tag, path)
             text = ""
         else:
             text += _process_text(tag)
@@ -341,9 +382,13 @@ def _get_href(element):
 
     return Href(url, text).dumps_as_content()
 
-def _generate_figure(document, element):
+def _generate_figure(document, element, path):
     image_tag = element.find("img")
-    image_url = image_tag.get("src")
+    if(image_tag != None):
+        image_url = image_tag.get("src")
+    else:
+        _process_content(document,element,path)
+        return
     caption_tag = element.find("div", { "class": "caption" })
 
     width = image_tag.get("alt")
@@ -355,18 +400,19 @@ def _generate_figure(document, element):
         end_index += start_index
 
     width = NoEscape(width[start_index:end_index])
-    image_path = NoEscape(_load_image(image_url))
+    image_path = NoEscape(_load_image(image_url, path).name)
 
-    text = ""
-    for tag in caption_tag:
-        text += _process_text(tag)
-    text = NoEscape(text)
+    if caption_tag != None:
+        text = ""
+        for tag in caption_tag:
+            text += _process_text(tag)
+        text = NoEscape(text)
 
     with document.create(Figure(position='h')) as fig:
         fig.add_image(image_path, width=width)
         fig.add_caption(text)
 
-def _generate_itemize(document, element, itemize=True):
+def _generate_itemize(document, element, path, itemize=True):
     enviroment = Itemize()
     if(not itemize):
         enviroment = Enumerate()
@@ -375,15 +421,15 @@ def _generate_itemize(document, element, itemize=True):
         for tag in filter(lambda x: x.name == "li", element):
             container = []
             for element in filter(lambda x: x != None, tag):
-                _process_content(container, element)
+                _process_content(container, element, path)
             itemize.add_item(dumps_list(container, escape=False, token=" "))
 
 
 
-def _generate_ilustration(document, element):
+def _generate_ilustration(document, element, path):
     image_tag = element.find("img")
     image_url = image_tag.get("src")
-    image_path = NoEscape(_load_image(image_url))
+    image_path = NoEscape(_load_image(image_url, path).name)
 
     description_tag = element.find("div", {"class": "description"})
     text = r"\footnotesize "
@@ -401,9 +447,9 @@ def _generate_ilustration(document, element):
         fig.add_image(image_path, width=width)
         fig.add_caption(text)
 
-def _generate_image(document, tag):
+def _generate_image(document, tag, path):
     url = tag.get("src")
-    path = NoEscape(_load_image(url))
+    path = NoEscape(_load_image(url, path).name)
 
     width = tag.get("alt")
     start_index = width.find("width=") + 6
@@ -430,20 +476,28 @@ def _generate_tabular(document, element):
         current_row =[]
         for column in filter(lambda x: x.name != None, row):
             text = ""
-            column = column.find("p")
-            if(column != None):
-                for tag in column:
-                    text += _process_text(tag)
-            current_row.append(text)
+
+            column_p = column.find("p")
+            if(column_p == None):
+                column_p = column
+
+            for tag in column_p:
+                text += _process_text(tag)
+
+            if(column.name == "th"):
+                current_row.append(bold(text))
+            else:
+                current_row.append(text)
         rows.append(current_row)
-    
     td = element.find("td")
+    if(td == None):
+        td = element.find("th")
     style = td.get("style")
 
     has_border_top = style.find("border-top") != -1
     has_border_sides = style.find("border-right") != -1
 
-    separator = ""
+    separator = " "
     if(has_border_sides):
         separator = "|"
 
@@ -474,7 +528,7 @@ def _generate_tabular(document, element):
 
         document.append(NewParagraph())
         document.append(tabular)
-def _process_content(document: Document, content: BeautifulSoup):
+def _process_content(document: Document, content: BeautifulSoup, path):
     text = ""
     for tag in content:
         if type(tag) == str:
@@ -482,7 +536,7 @@ def _process_content(document: Document, content: BeautifulSoup):
         elif tag.name == "p":
             document.append(NoEscape(text))
             text = ""
-            _generate_paragraph(document, tag)            
+            _generate_paragraph(document, tag, path)            
         elif tag.name == "span" and "tex2jax_process" in tag.get("class"):
             document.append(NoEscape(text))
             text = ""
@@ -498,20 +552,20 @@ def _process_content(document: Document, content: BeautifulSoup):
         elif tag.name == "div" and "figure" in tag.get("class"):
             document.append(NoEscape(text))
             text = ""
-            _generate_figure(document, tag)
+            _generate_figure(document, tag, path)
         elif tag.name == "div" and "illustration" in tag.get("class"):
             document.append(NoEscape(text))
             text = ""
-            _generate_ilustration(document, tag)
+            _generate_ilustration(document, tag, path)
         elif tag.name == "img":
             document.append(NoEscape(text))
             text = ""
-            _generate_image(document, tag)
+            _generate_image(document, tag, path)
         elif tag.name == "center":
             document.append(NoEscape(text))
             text = ""
             center = Center()
-            _process_content(center, tag)
+            _process_content(center, tag, path)
             document.append(center)
         elif tag.name == "blockquote":
             document.append(NoEscape(text))
@@ -524,16 +578,28 @@ def _process_content(document: Document, content: BeautifulSoup):
         elif tag.name == "ul" or tag.name == "ol":
             document.append(NoEscape(text))
             text = ""
-            _generate_itemize(document, tag, tag.name == "ul")
+            _generate_itemize(document, tag, path, tag.name == "ul")
         elif tag.name == "table" and "sample" in tag.get("class"):
             document.append(NoEscape(text))
             text = ""
-            _generate_sample(document, tag)            
+            _generate_sample(document, tag, path)        
+        elif tag.name == "div" and "minipage" in tag.get("class"):
+            document.append(NoEscape(text))
+            text = ""
+            _generate_minipage(document, tag, path)  
+        elif tag.name == "div" and "sampleinteractionwrite" in tag.get("class"):
+            document.append(NoEscape(text))
+            text = ""
+            _generate_interactive_write(document, tag, path)  
+        elif tag.name == "div" and "sampleinteractionread" in tag.get("class"):
+            document.append(NoEscape(text))
+            text = ""
+            _generate_interactive_read(document, tag, path)  
         else:
             text += _process_text(tag)
     document.append(NoEscape(text))
 
-def _generate_sample(document, element):
+def _generate_sample(document, element, path):
     global sample_number
     if(sample_number == 0):
         document.append(Subsection(f"Sample", numbering=False))
@@ -544,20 +610,36 @@ def _generate_sample(document, element):
     elements = []
     for td in element.find_all("td"):
         content = []
-        _process_content(content, td)
+        _process_content(content, td, path)
         elements.append(content)
 
     for element in elements:
         document.append(Tcolorbox(dumps_list(element, escape=False, token=" ") + NoEscape("\n")))
 
-def _generate_content(problem: str, document: Document, parser: BeautifulSoup, metadata):
+def _generate_interactive_write(document, element, path):
+    content = []
+    _process_content(content, element, path)
+    document.append(Tcolorbox(dumps_list(content, escape=False, token=" ") + NoEscape("\n"), """colback=lime!10, grow to left by=-10mm"""))
+
+def _generate_interactive_read(document, element, path):
+    content = []
+    _process_content(content, element, path)
+    document.append(Tcolorbox(dumps_list(content, escape=False, token=" ") + NoEscape("\n"), """colback=yellow!10, grow to right by=-10mm"""))
+
+def _generate_content(problem: str, document: Document, parser: BeautifulSoup, metadata, path):
     content = parser.find("div", { "book-page-fixed_width"})
     title = escape_latex(content.find("h1").get_text())
     title += r" \mdseries\large" + f"{metadata.memory_limit}, {metadata.time_limit}"
     document.append(Section(NoEscape(title), numbering=False))
 
-    content = content.find("div", { "class": "problembody" })
-    _process_content(document, content)
+    body = content.find("div", { "class": "problembody" })
+    _process_content(document, body, path)
+
+    foot = content.find("div", { "class": "footnotes" })
+
+    if(foot != None):
+        _process_content(document, foot, path)
+
 
 def _get_document() -> Document:
     # Geometry setup
@@ -571,10 +653,6 @@ def _get_document() -> Document:
 
     document = Document(geometry_options=geometry_settings)
 
-    # Hypereference setup
-    hyperref_config = Hypersetup(colorlinks=True, urlcolor="black")
-    document.preamble.append(hyperref_config)
-
     # Extra packages setup
     packages = [
         ("wrapfig", None),
@@ -589,6 +667,12 @@ def _get_document() -> Document:
     for package, options in packages:
         document.preamble.append(Package(package, options=options))
 
+    # Hypereference setup
+    hyperref_config = Hypersetup(colorlinks=True, urlcolor="black")
+    document.preamble.append(hyperref_config)
+
+
+
     # Wrapfigure sep
     document.preamble.append(NoEscape(r"\setlength{\intextsep}{0pt}"))
 
@@ -597,17 +681,17 @@ def _get_document() -> Document:
 def generate_pdf(problem: str, path = None):
     global sample_number
     try:
+        if(path == None):
+            path = Path(problem)
+        else:
+            path = path.with_suffix("")
+
         sample_number = 0    
         parser = _get_html(problem)
         document = _get_document()
         metadata = _generate_header(problem, document, parser)
-        _generate_content(problem, document, parser, metadata)
+        _generate_content(problem, document, parser, metadata, path.parent)
         
-        if(path == None):
-            path = problem
-        else:
-            path = path.with_suffix("")
-
         document.generate_pdf(clean_tex=True, filepath=path)
     
     except KeyboardInterrupt:
@@ -618,8 +702,6 @@ def generate_pdf(problem: str, path = None):
             except (FileNotFoundError):
                 pass
     finally:
-        for path in _files_to_remove:
-            os.remove(path)
-        _files_to_remove.clear()
-
-    
\ No newline at end of file
+        for file in _files_to_remove:
+            file.unlink()
+        _files_to_remove.clear()
\ No newline at end of file
